% This codes computes the impulse responses for the three cases and store
% the values of the loss function, the volatilities of the target variables
% and the values of the optimized parameters.

% ************************************
% * Technology shock                 *
% * only L/Y in macroprudential rule *
% * zero weight on output            *
% ************************************

clear all;
clc;

randn('seed',1566);  
rand('seed',1234); 

weight_Y_CB  = 0.5;
weight_PI_CB = 1.0;
weight_R_CB  = 0.1;

weight_Y_MP  = 0.0;
weight_LY_MP = 1.0;
weight_V_MP  = 0.1;

% **********************
% ** cooperative case **
% **********************

cd cooperative;

load coop_optimized.txt;

thetstar = coop_optimized;

loss_MP_FSP;

loss_coop     = loss;

loss_MP_coop  = weight_Y_CB*oo_.var(3,3)*100 + weight_PI_CB*oo_.var(14,14)*100 + weight_R_CB*oo_.var(17,17)*100;

loss_FSP_coop = weight_Y_MP*oo_.var(3,3)*100 + weight_LY_MP*oo_.var(18,18)*100 + weight_V_MP*oo_.var(16,16)*100;

losses_coop = [ loss_coop, loss_MP_coop, loss_FSP_coop ] ;

stddev_FSP_MP = 100.*(diag(oo_.var)).^0.5;

irf_Y_c  = oo_.irfs.Y1_e_A_e;
irf_C_c  = oo_.irfs.C_e_A_e;
irf_I_c  = oo_.irfs.I_e_A_e;
irf_L_c  = oo_.irfs.L_e_A_e;
irf_LY_c = oo_.irfs.L_Y_e_A_e;
irf_vi_c = oo_.irfs.vi_e_A_e;
irf_r_c  = oo_.irfs.r_ib_e_A_e;
irf_pi_c = oo_.irfs.pie_e_A_e;
irf_Kr_c = oo_.irfs.k_ratio_e_A_e;
irf_rb_c = oo_.irfs.r_be_e_A_e;
irf_rd_c = oo_.irfs.r_d_e_A_e;
irf_Kb_c = oo_.irfs.K_b_e_A_e;
irf_rr_c = (oo_.irfs.r_ib_e_A_e - oo_.irfs.pie_e_A_e);

irfs_c = [ irf_Y_c', irf_L_c', irf_LY_c', irf_vi_c', irf_pi_c', irf_Kr_c', irf_rb_c', irf_Kb_c', irf_r_c' ];

save impulses_c.txt   irfs_c          -ASCII -DOUBLE;
save variances_c.txt  stddev_FSP_MP   -ASCII -DOUBLE;
save losses_c.txt     losses_coop     -ASCII -DOUBLE;

clear all;

weight_Y_CB  = 0.5;
weight_PI_CB = 1.0;
weight_R_CB  = 0.1;

weight_Y_MP  = 0.0;
weight_LY_MP = 1.0;
weight_V_MP  = 0.1;

% **************************
% ** non-cooperative case **
% **************************

cd ..;
cd non_cooperative;

load optimized_coeffs_nc.txt;

thetstar_fsp = optimized_coeffs_nc(1:2);
thetstar_mp  = optimized_coeffs_nc(3:5);

loss_FSP;

loss_FSP_nc = loss;

loss_MP;

loss_MP_nc = loss;

losses_non_coop = [ loss_FSP_nc + loss_MP_nc, loss_MP_nc, loss_FSP_nc ] ;

stddev_nc = 100.*(diag(oo_.var)).^0.5;

irf_Y_nc  = oo_.irfs.Y1_e_A_e;
irf_C_nc  = oo_.irfs.C_e_A_e;
irf_I_nc  = oo_.irfs.I_e_A_e;
irf_L_nc  = oo_.irfs.L_e_A_e;
irf_LY_nc = oo_.irfs.L_Y_e_A_e;
irf_vi_nc = oo_.irfs.vi_e_A_e;
irf_r_nc  = oo_.irfs.r_ib_e_A_e;
irf_pi_nc = oo_.irfs.pie_e_A_e;
irf_Kr_nc = oo_.irfs.k_ratio_e_A_e;
irf_rb_nc = oo_.irfs.r_be_e_A_e;
irf_rd_nc = oo_.irfs.r_d_e_A_e;
irf_Kb_nc = oo_.irfs.K_b_e_A_e;
irf_rr_nc = (oo_.irfs.r_ib_e_A_e - oo_.irfs.pie_e_A_e);

irfs_nc = [ irf_Y_nc', irf_L_nc', irf_LY_nc', irf_vi_nc', irf_pi_nc', irf_Kr_nc', irf_rb_nc', irf_Kb_nc', irf_r_nc' ];

save impulses_nc.txt   irfs_nc          -ASCII -DOUBLE;
save variances_nc.txt  stddev_nc        -ASCII -DOUBLE;
save losses_nc.txt     losses_non_coop  -ASCII -DOUBLE;

clear all;

weight_Y_CB  = 0.5;
weight_PI_CB = 1.0;
weight_R_CB  = 0.1;

weight_Y_MP  = 0.0;
weight_LY_MP = 1.0;
weight_V_MP  = 0.1;

% ****************************
% Monetary policy only case **
% ****************************

cd ..;
cd mon_pol;

load mp_optimized.txt;

thetstar = mp_optimized;

loss_MP;

loss_only_mp     = loss;

loss_FSP_only_mp = weight_Y_MP*oo_.var(3,3)*100 + weight_LY_MP*oo_.var(18,18)*100 + weight_V_MP*oo_.var(16,16)*100;

losses_only_mon_pol = [ loss_FSP_only_mp + loss_only_mp, loss_only_mp, loss_FSP_only_mp ] ;

stddev_only_mp = 100.*(diag(oo_.var)).^0.5;

irf_Y_only_mp  = oo_.irfs.Y1_e_A_e;
irf_C_only_mp  = oo_.irfs.C_e_A_e;
irf_I_only_mp  = oo_.irfs.I_e_A_e;
irf_L_only_mp  = oo_.irfs.L_e_A_e;
irf_LY_only_mp = oo_.irfs.L_Y_e_A_e;
irf_r_only_mp  = oo_.irfs.r_ib_e_A_e;
irf_pi_only_mp = oo_.irfs.pie_e_A_e;
irf_Kr_only_mp = oo_.irfs.k_ratio_e_A_e;
irf_rb_only_mp = oo_.irfs.r_be_e_A_e;
irf_rd_only_mp = oo_.irfs.r_d_e_A_e;
irf_Kb_only_mp = oo_.irfs.K_b_e_A_e;
irf_rr_only_mp = (oo_.irfs.r_ib_e_A_e - oo_.irfs.pie_e_A_e);
irf_vi_only_mp = zeros(length(irf_rr_only_mp),1)';

irfs_only_mp = [ irf_Y_only_mp', irf_L_only_mp', irf_LY_only_mp', irf_vi_only_mp', irf_pi_only_mp', irf_Kr_only_mp', irf_rb_only_mp', irf_Kb_only_mp', irf_r_only_mp' ];

save impulses_only_mp.txt   irfs_only_mp          -ASCII -DOUBLE;
save variances_only_mp.txt  stddev_only_mp        -ASCII -DOUBLE;
save losses_only_mp.txt     losses_only_mon_pol   -ASCII -DOUBLE;

clear all;

clc;

% *************
% ** Results **
% *************

% cd d:\dati\papers_miei\paper_Basel_II\July_2011\technology_shock\check\robustness\rule\zero\cooperative;
cd ..;
cd cooperative;

load impulses_c.txt;
load variances_c.txt;
load losses_c.txt;
load coop_optimized.txt;

coop_optimized(2) = (coop_optimized(2)^2)/(0.99+coop_optimized(2)^2);
coop_optimized(3) = (coop_optimized(3)^2)/(0.99+coop_optimized(3)^2);

% cd d:\dati\papers_miei\paper_Basel_II\July_2011\technology_shock\check\robustness\rule\zero\non_cooperative;
cd ..;
cd non_cooperative;

load impulses_nc.txt;
load variances_nc.txt;
load losses_nc.txt;
load optimized_coeffs_nc.txt;

optimized_coeffs_nc(2) = (optimized_coeffs_nc(2)^2)/(0.99+optimized_coeffs_nc(2)^2);
optimized_coeffs_nc(3) = (optimized_coeffs_nc(3)^2)/(0.99+optimized_coeffs_nc(3)^2);

% cd D:\Dati\papers_miei\paper_Basel_II\July_2011\technology_shock\check\robustness\rule\zero\mon_pol;
cd ..;
cd mon_pol;

load impulses_only_mp.txt;
load variances_only_mp.txt;
load losses_only_mp.txt;
load mp_optimized.txt;

mp_optimized(1) = (mp_optimized(1)^2)/(0.99+mp_optimized(1)^2);

% cd d:\dati\papers_miei\paper_Basel_II\July_2011\technology_shock\check\robustness\rule\zero;
cd ..;


% *******************************************
% compare volatilities, losses and parameters
% *******************************************

table_volatilities = [ variances_c([14,3,18,17,16]), variances_nc([14,3,18,17,16]), variances_only_mp([14,3,18,17,16]) ]';

table_losses =  [ losses_c; losses_nc; losses_only_mp ]';

table_params = [ coop_optimized;  optimized_coeffs_nc; [ 0, 0, mp_optimized ] ];

% ****************
% plotting results
% ****************

vi_ss   = 0.09;
r_ib_ss = 0.00982744500797393;
r_be_ss = 0.0156082950126645;
LY_ss   = 2.63528153779441;

figure;

fs_axis     = 12;
fs_legend   = 12;

set(gcf,'Color',[1 1 1]); % White background

subplot(4,2,1);

plot((vi_ss-vi_ss.*impulses_c(:,4)).*100,'Linewidth',4);
hold on
plot((vi_ss-vi_ss.*impulses_nc(:,4)).*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot((vi_ss-vi_ss.*impulses_only_mp(:,4)).*100,'g--','Linewidth',3);
title('Capital requirements','Fontsize',14);
ylim([8.8 9.4]);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,2);

plot(-impulses_c(:,9).*400.*r_ib_ss,'Linewidth',4);
hold on
plot(-impulses_nc(:,9).*400.*r_ib_ss,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,9).*400.*r_ib_ss,'g--','Linewidth',3);
title('Policy rate','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,3);

plot(-impulses_c(:,1).*100,'Linewidth',4);
hold on
plot(-impulses_nc(:,1).*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,1).*100,'g--','Linewidth',3);
title('Output','Fontsize',14);
legend('Cooperative','Non cooperative','Only monetary policy');
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,4);

plot(-impulses_c(:,3)*LY_ss.*100,'Linewidth',4);
hold on
plot(-impulses_nc(:,3)*LY_ss.*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,3)*LY_ss.*100,'g--','Linewidth',3);
title('Loans-to-output ratio','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,5);

plot(-impulses_c(:,7).*400.*r_be_ss,'Linewidth',4);
hold on
plot(-impulses_nc(:,7).*400.*r_be_ss,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,7).*400.*r_be_ss,'g--','Linewidth',3);
title('Interest rate on loans','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,6);

plot(-impulses_c(:,2).*100,'Linewidth',4);
hold on
plot(-impulses_nc(:,2).*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,2).*100,'g--','Linewidth',3);
title('Loans','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,7);

plot(-impulses_c(:,5).*100,'Linewidth',4);
hold on
plot(-impulses_nc(:,5).*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot(-impulses_only_mp(:,5).*100,'g--','Linewidth',3);
title('Inflation','Fontsize',14);
xlabel('quarters after shock','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

subplot(4,2,8);

plot((vi_ss-vi_ss.*impulses_c(:,8)).*100,'Linewidth',4);
hold on
plot((vi_ss-vi_ss.*impulses_nc(:,8)).*100,'r-o','Linewidth',2,'MarkerSize',5,'MarkerFaceColor','r');
hold on
plot((vi_ss-vi_ss.*impulses_only_mp(:,8)).*100,'g--','Linewidth',3);
title('Capital-assets ratio','Fontsize',14);
ylim([8.8 9.2001]);
xlabel('quarters after shock','Fontsize',14);
grid;
set(gca,'FontWeight','normal','FontSize',fs_axis);

