% ******************************************************************************************************* 
% Case with technology shocks: mod file with variables, equations and steady state initial conditions, **
% a series of check for the solution and computation of the loss function                              **
% *******************************************************************************************************

% ******************************************************************************************************************************
% Non-cooperative case: the .mod file is the same as loss_MP_FSP.mod, the only difference is the definition of the loss function
% ******************************************************************************************************************************

% **************************************************************************************************************
% This .mod file generates a .m file that must be modified to be used in optimisation; see file loss_MP_FSP.m **
% **************************************************************************************************************

% *********************************
% THIS VERSION: 24 FEBRUARY 2014 **
% *********************************

% ************************************
% Definition of the model variables **   
% ************************************

var 

c_p       // 		1  
h_p       // 		2  
d_p       // 		3  
l_p       // 		4  
lam_p     // 		5  
J_R       // 		6  
j_B       // 		7  
pie_wp    // 		8  
c_i       // 		9  
h_i       // 		10 
b_i       // 		11 
l_i       // 		12 
lam_i     // 		13 
s_i       // 		14 
pie_wi	  //		15 
I		  //		16 
q_k		  //		17 
c_e		  //		18 
k_e		  //		19 
l_pd	  //		20 
l_id	  //		21 
b_ee	  //		22 
y_e		  //		23 
lam_e	  //		24 
s_e		  //		25 
d_b		  //		26 
b_h		  //		27 
b_e		  //		28 
r_d		  //		29 
r_bh	  //		30 
r_be	  //		31 
R_b		  //		32 
K_b		  //		33 
pie       // 		34 
x         // 		35 
C         // 		36 
Y         // 		37 
D         // 		38 
BE		  //		39 
BH		  //		40 
B         // 		41 
w_p       // 		42 
w_i       // 		43 
J_B		  //		44 
q_h       // 		45 
K		  //		46 
PIW		  //		47 
r_ib      // 		48 
r_k       // 		49 
ee_z      // 		50 
A_e       // 		51 
ee_j      // 		52 
mk_d      // 		53 
mk_be     // 		54 
mk_bh     // 		55 
ee_qk     // 		56 
m_i       // 		57 
m_e       // 		58 
eps_y     // 		59 
eps_l     // 		60 
eps_K_b   // 		61 
Y1		  //		62 
spr_b	  //		63 
rr_e	  //		64 
aux1      //		65 
vi		  //		66 
L	      //		67 
weight_BH //		68 
weight_BE //		69 
P_S       //		70 
k_ratio   //		71 
d_r       //        72
d_vi      //        73
L_Y;      //        74

% *************************************
% Definition of the exogenous variables
% *************************************

varexo e_A_e e_eps_K_b e_j e_l e_me e_mi e_mk_be e_mk_bh e_mk_d e_r_ib e_qk e_y e_z;  

parameters  beta_p j phi beta_i m_i_ss beta_e m_e_ss alpha                  // HOUSEHOLDS & ENTREPRENEURS
            h a_i a_p a_e gamma_p gamma_i gamma_e ni                        // HOUSEHOLDS & ENTREPRENEURS
            eps_l_ss kappa_w                                                // HOUSEHOLDS (labor params)
            eps_d eps_bh eps_be                                             // BANKS 
            mk_d_ss mk_bh_ss mk_be_ss r_be_ss  r_bh_ss r_k_ss               // BANKS
            gamma_b beta_b delta_kb kappa_kb                                // BANKS 
            eps_y_ss kappa_p ind_p ind_w                                    // RETAILERS
            kappa_i kappa_d kappa_be kappa_bh deltak                        // OTHERS
            piss  r_ib_ss vi_ss                                             // STEADY STATE
            rho_ee_z rho_A_e rho_ee_j rho_mi rho_me rho_eps_y               // SHOCKS
            rho_mk_d rho_mk_be rho_mk_bh rho_ee_qk rho_eps_l rho_eps_K_b    // SHOCKS
            rho_vi chi_vi1 chi_vi2 rhovi rhoib rho_ib phi_pie phi_y;        // PARAMETERS OF MONETARY AND MACROPRU RULES
            
% **********************************************************************************************************************            
% estimated parameters are set at the median of the posterior marginal distributions computed in Gerali et al. (2010) **
% **********************************************************************************************************************

beta_p     	=	0.994300000000000       ; % calibrated
j          	=	0.200000000000000       ; % calibrated
phi        	=	1.000000000000000   	; % calibrated
beta_i     	=	0.975000000000000       ; % calibrated
m_i_ss     	=	0.700000000000000       ; % calibrated
beta_e     	=	0.975000000000000   	; % calibrated
m_e_ss     	=	0.350000000000000       ; % calibrated
alpha      	=	0.300000000000000       ; % calibrated
h          	=	1.000000000000000   	; % calibrated
a_p        	=	0.000000000000000   	; % calibrated
a_e        	=	0.000000000000000       ; % calibrated
gamma_p    	=	1.000000000000000       ; % calibrated
gamma_i    	=	1.000000000000000       ; % calibrated
gamma_e    	=	1.000000000000000       ; % calibrated
ni         	=	0.800000000000000       ; % calibrated
eps_l_ss   	=	5.000000000000000   	; % calibrated
eps_d      	=	-1.4                    ; % calibrated
eps_bh     	=	2.7                     ; % calibrated
eps_be     	=	2.7                     ; % calibrated
mk_d_ss    	=	eps_d   / (eps_d  - 1)  ; % calibrated
mk_bh_ss   	=	eps_bh  / (eps_bh - 1)  ; % calibrated
mk_be_ss   	=	eps_be  / (eps_be - 1)  ; % calibrated
gamma_b    	=	1.000000000000000       ; % calibrated
beta_b     	=	0.994300000000000   	; % calibrated
deltak     	=	0.025000000000000       ; % calibrated
piss       	=	1.000000000000000       ; % calibrated
vi_ss      	=	0.090000000000000   	; % calibrated
eps_y_ss   	=	6.000000000000000   	; % calibrated
h           =   1                       ; % calibrated                                  
a_i        	=	0.8560                  ; % estimated 
kappa_w    	=	99.8983                 ; % estimated
kappa_kb   	=	11.0683             	; % estimated
kappa_p    	=	28.6502             	; % estimated
ind_p      	=	0.1605              	; % estimated
ind_w      	=	0.2757              	; % estimated
kappa_i    	=	10.1822             	; % estimated
kappa_d    	=	3.5030              	; % estimated
kappa_be   	=	9.3638                	; % estimated
kappa_bh   	=	10.0867             	; % estimated
rho_ee_z   	=	0.97390             	; % estimated
rho_A_e    	=	0.97390             	; % estimated
rho_ee_j   	=	0.90922                 ; % estimated
rho_mi     	=	0.93857              	; % estimated
rho_me     	=	0.91017              	; % estimated
rho_eps_y  	=	0.25629              	; % estimated
rho_mk_d   	=	0.85949                 ; % estimated
rho_mk_be  	=	0.81010              	; % estimated
rho_mk_bh  	=	0.77728              	; % estimated
rho_ee_qk  	=	0.44099              	; % estimated
rho_eps_l  	=	0.67703              	; % estimated
rho_eps_K_b	=	0.81505              	; % estimated

% ************
% calculations
% ************

r_ib_ss     = (piss/beta_p - 1) * (eps_d-1)/eps_d;     
r_be_ss     = r_ib_ss*eps_be/(eps_be-1) ;				
r_bh_ss     = r_ib_ss*eps_bh/(eps_bh-1) ;				
r_k_ss      = -(1-deltak)-m_e_ss*(1-deltak)*piss/beta_e*(1/(1+r_be_ss)-beta_e/piss)+1/beta_e;
eksi_1      = r_k_ss;                             
eksi_2      = 0.1*r_k_ss;                         
eps_b       = mean([eps_bh,eps_be]);
delta_kb    = r_ib_ss/vi_ss * (eps_d - eps_b + vi_ss * eps_d*(eps_b-1))/((eps_b-1)*(eps_d-1));  

% ***************************************
% initialising parameters of policy rules
% ***************************************

chi_vi1     = 0.98256313039023;
chi_vi2     = 0;
rhovi       = 1;
rho_vi     	= (rhovi^2)/(0.99+rhovi^2);
rhoib     	= 1;
rho_ib     	= (rhoib^2)/(0.99+rhoib^2);
phi_pie    	= 2.22753795035858;
phi_y      	= 1.00557199044067;

% *****************************
% definition of model equations
% *****************************

model;

%**************************************
%*********** PATIENT HHs **************
%**************************************

%// Eq. A1

exp(c_p) + exp(q_h) * ( exp(h_p) - exp(h_p(-1)) ) + exp(d_p) = exp(w_p) * exp(l_p) + (1+exp(r_d(-1)))*exp(d_p(-1))/exp(pie) + exp(J_R)/gamma_p ;  //

%// Eq. A2

(1-a_i)*exp(ee_z)*(exp(c_p) - a_i*exp(c_p(-1)))^(-1) = exp(lam_p);

%// Eq. A3

j * (exp(ee_j)) / exp(h_p) - exp(lam_p) * exp(q_h) + beta_p * exp(lam_p(+1)) * exp(q_h(+1)) = 0; 

%// Eq. A4

exp(lam_p) = beta_p * exp(lam_p(+1)) * (1+exp(r_d)) / exp(pie(+1));


%*****************************************************
%*********** IMPATIENT HHs ***************************
%*****************************************************

%// Eq. A5

exp(c_i) + exp(q_h) * (exp(h_i) - exp(h_i(-1))) + (1+exp(r_bh(-1)))*exp(b_i(-1))/exp(pie) = exp(w_i) * exp(l_i) + exp(b_i);

%// Eq. A6

(1+exp(r_bh)) * exp(b_i) = exp(m_i) * exp(q_h(+1)) * exp(h_i) * exp(pie(+1));

%// Eq. A7

(1-a_i)*exp(ee_z)*(exp(c_i) - a_i*exp(c_i(-1)))^(-1)  = exp(lam_i); 

%// Eq. A8

j * (exp(ee_j))  / exp(h_i) - exp(lam_i) * exp(q_h) + beta_i * exp(lam_i(+1)) * exp(q_h(+1))  + exp(s_i) * exp(m_i) *exp(q_h(+1)) * exp(pie(+1))  = 0;

%// Eq. A9

exp(lam_i) - beta_i * exp(lam_i(+1)) * (1+exp(r_bh)) / exp(pie(+1)) = exp(s_i) * (1+exp(r_bh));

%**************************************
%************  ENTREPRENEURS **********
%**************************************

%// Eq. A10

exp(c_e) + ((1+exp(r_be(-1))) * exp(b_ee(-1)) / exp(pie) ) +  (exp(w_p)*exp(l_pd) + exp(w_i)*exp(l_id)) + exp(q_k) * exp(k_e) = 
    exp(y_e) / exp(x) + exp(b_ee) + exp(q_k) * (1-deltak) * exp(k_e(-1));

%// Eq. A11

(1+exp(r_be)) * exp(b_ee) = exp(m_e) * exp(q_k(+1)) * exp(pie(+1)) * exp(k_e) * (1-deltak);

%// Eq. A12

(1-a_i)*(exp(c_e) - a_i*exp(c_e(-1)))^(-1) = exp(lam_e);

%// Eq. A13

exp(lam_e) - exp(s_e)  * (1+exp(r_be)) = beta_e * exp(lam_e(+1)) * (1+exp(r_be)) / exp(pie(+1));

%// Eq. A14

exp(s_e) * exp(m_e) * exp(q_k(+1)) * exp(pie(+1)) * (1-deltak) + beta_e * exp(lam_e(+1)) * ( exp(q_k(+1))*(1-deltak) + exp(r_k(+1)) ) = exp(lam_e) * exp(q_k);

%// Eq. A15

exp(y_e) = exp(A_e) * (exp(k_e(-1)))^(alpha) * ( exp(l_pd)^ni * exp(l_id)^(1-ni) ) ^ (1-alpha);

%// Eq. A16

exp(w_p) =  ni  * (1-alpha) * exp(y_e) / ( exp(l_pd) * exp(x) );

%// Eq. A17

exp(w_i) = (1-ni) * (1-alpha) * exp(y_e) / ( exp(l_id) * exp(x) );
		  
%// Eq. A18
		  
exp(r_k) = alpha * exp(A_e) * exp(k_e(-1))^(alpha-1) * ( exp(l_pd)^ni * exp(l_id)^(1-ni) ) ^ (1-alpha) /exp(x);

%**************************************************************************
%***********  CAPITAL PRODUCERS *******************************************
%**************************************************************************

%// Eq. A19

exp(K) = (1-deltak) * exp(K(-1)) + ( 1 - kappa_i/2 * (exp(I)*exp(ee_qk)/exp(I(-1)) - 1)^2 ) * exp(I) ;   

%// Eq. A20

1 = exp(q_k) * ( 1 -  kappa_i/2 * (exp(I)*exp(ee_qk)/exp(I(-1)) - 1)^2  - kappa_i * (exp(I)*exp(ee_qk)/exp(I(-1)) - 1) * exp(I)*exp(ee_qk)/exp(I(-1)) ) 
  + beta_e * exp(lam_e(+1)) / exp(lam_e) * exp(q_k(+1)) *   kappa_i * (exp(I(+1))*exp(ee_qk(+1))/exp(I) - 1) * exp(ee_qk(+1)) * (exp(I(+1))/exp(I))^2 ; //   13                              //   15

%***************************************************  
%*********** RETAILERS *****************************
%***************************************************

%// Eq. A21

exp(J_R)  = exp(Y)*(1 - (1/exp(x))    - (kappa_p/2) * (exp(pie) - ( exp(pie(-1)) ^ ind_p * piss ^ (1-ind_p) ))^2 );

%// Eq. A22

1 - exp(eps_y) + exp(eps_y) / exp(x) -    kappa_p * (exp(pie)     - ( exp(pie(-1)) ^ ind_p * piss ^ (1-ind_p) )) * exp(pie) 
    + beta_p*(exp(lam_p(+1))/exp(lam_p))* kappa_p * (exp(pie(+1)) - ( exp(pie)     ^ ind_p * piss ^ (1-ind_p) )) * exp(pie(+1)) * (exp(Y(+1))/exp(Y)) = 0;

%// Eq. A23

(1 - exp(eps_l)) * exp(l_p) + exp(l_p) ^(1+phi) / exp(w_p) * exp(eps_l)/exp(lam_p) 
                                         - kappa_w *( exp(pie_wp)     - exp(pie(-1)) ^ ind_w * piss ^ (1-ind_w) ) * exp(pie_wp)
   +  beta_p * exp(lam_p(+1))/exp(lam_p) * kappa_w *( exp(pie_wp(+1)) - exp(pie)     ^ ind_w * piss ^ (1-ind_w) ) * exp(pie_wp(+1)) ^2 / exp(pie) = 0 ;

%// Eq. A24

exp(pie_wp) = exp(w_p) / exp(w_p(-1)) * exp(pie);

%// Eq. A25

(1 - exp(eps_l)) * exp(l_i) + exp(l_i) ^(1+phi) / exp(w_i) * exp(eps_l)/exp(lam_i) 
                                         - kappa_w *( exp(pie_wi)     - exp(pie(-1))^ind_w * piss ^ (1-ind_w) ) * exp(pie_wi)
   +  beta_i * exp(lam_i(+1))/exp(lam_i) * kappa_w *( exp(pie_wi(+1)) - exp(pie)    ^ind_w * piss ^ (1-ind_w) ) * exp(pie_wi(+1)) ^2 / exp(pie) = 0;

%// Eq. A26

exp(pie_wi) = exp(w_i) / exp(w_i(-1)) * exp(pie);

%*************************************
%*************  BANKS ****************
%*************************************

%// Eq. A27

- 1 + exp(mk_d) /(exp(mk_d)-1)   - exp(mk_d)/(exp(mk_d)-1)  * exp(r_ib)/exp(r_d)  - kappa_d  * ( exp(r_d)/exp(r_d(-1)) - 1  )  * exp(r_d)/exp(r_d(-1)) 
  + beta_p * ( exp(lam_p(+1))/exp(lam_p) ) * kappa_d  * ( exp(r_d(+1))/exp(r_d) - 1 )   * ( (exp(r_d(+1))/exp(r_d))^2 )   * (exp(d_b(+1))/exp(d_b)) = 0;// 

%// Eq. A28

+ 1 - exp(mk_be)/(exp(mk_be)-1)  +  exp(mk_be)/(exp(mk_be)-1)  * exp(R_b)/exp(r_be) - kappa_be * (exp(r_be)/exp(r_be(-1)) - 1 ) * exp(r_be)/exp(r_be(-1)) 
  + beta_p * ( exp(lam_p(+1))/exp(lam_p) ) * kappa_be * ( exp(r_be(+1))/exp(r_be) - 1 ) * ( (exp(r_be(+1))/exp(r_be))^2 ) * (exp(b_e(+1))/exp(b_e)) = 0;//   30

%// Eq. A29
  
+ 1 - exp(mk_bh)/(exp(mk_bh)-1)  +  exp(mk_bh)/(exp(mk_bh)-1)  * exp(R_b)/exp(r_bh) - kappa_bh * (exp(r_bh)/exp(r_bh(-1)) - 1 ) * exp(r_bh)/exp(r_bh(-1)) 
  + beta_p * ( exp(lam_p(+1))/exp(lam_p) ) * kappa_bh * ( exp(r_bh(+1))/exp(r_bh) - 1 ) * ( (exp(r_bh(+1))/exp(r_bh))^2 ) * (exp(b_h(+1))/exp(b_h)) = 0;//

%// Eq. A30
  
exp(R_b) = - kappa_kb * ( exp(K_b) / exp(B) - exp(vi) ) * ( exp(K_b)/exp(B) ) ^ 2  + exp(r_ib) ; 

%// Eq. A31

exp(K_b) * exp(pie) = (1-delta_kb) * exp(K_b(-1))  + exp(j_B(-1)) ;

%// Eq. A32

exp(b_h) + exp(b_e) = exp(d_b) + exp(K_b);

exp(d_b)  = exp(d_p) ;                                      
exp(b_h)  = exp(b_i) ; 
exp(b_e)  = exp(b_ee); 

% Eq. A33

exp(weight_BH) = ( 1 ^ ( 1-0.94 ) ) * ( exp(Y1) / exp(Y1(-4)) )^( ( 1 - 0.94 ) * ( - 10 * 1 ) ) * exp( weight_BH(-1) )^0.94;

% Eq. A34

exp(weight_BE) = ( 1 ^ ( 1-0.92 ) ) * ( exp(Y1) / exp(Y1(-4)) )^( ( 1 - 0.92 ) * ( - 15 * 1 ) ) * exp( weight_BE(-1) )^0.92;

% Eq. A35

exp(j_B) = + exp(r_bh)  *  exp(b_h)
           + exp(r_be)  *  exp(b_e) 
           - exp(r_d)   *  exp(d_b)           
           - kappa_d/2  * ( (exp(r_d)/exp(r_d(-1))-1)^2)   * exp(r_d) *exp(d_b) 
           - kappa_be/2 * ( (exp(r_be)/exp(r_be(-1))-1)^2) * exp(r_be)*exp(b_e) 
           - kappa_bh/2 * ( (exp(r_bh)/exp(r_bh(-1))-1)^2) * exp(r_bh)*exp(b_h)
           - kappa_kb/2 * ( ( exp(K_b) / exp(B)  - exp(vi) ) ^ 2) * exp(K_b) ; 

%*************************************
%*********** Monetary policy *********
%*************************************

% Eq. A36

(1+exp(r_ib)) = (1+r_ib_ss)^(1-rho_ib) * (1+exp(r_ib(-1)))^rho_ib * 
                ( ( exp(pie) / piss) ^phi_pie * (exp(Y1)/exp(Y1(-1)))^phi_y  ) ^ (1-rho_ib) * (1+e_r_ib) ;//   48

%******************************************
%*********** Capital requirements *********
%******************************************

% Eq. A37

exp(vi)        = ( vi_ss ^ ( 1-rho_vi ) ) * ( exp(L_Y) / exp(steady_state(L_Y)) )^( ( chi_vi1 * 1  ) ) * ( exp(Y1) / exp(Y1(-1)) )^( ( chi_vi2 * 1  ) )* exp( vi(-1) )^rho_vi;
	   
    	   
% NOT INCLUDED IN THE MODEL

exp(P_S) * exp(lam_p) = beta_p * exp(lam_p(+1)) * ( exp( P_S(+1) ) + exp(J_R)   ) / exp(pie(+1));

% *******************************
% Market clearing and definitions
% *******************************

% Eq. A38 - Definition of ouptut

exp(Y1)             = exp(C) + 1 * (exp(K)-(1-deltak)*exp(K(-1)));

% This is the market clearing condition which can be excldued by Walras' law

%//exp(Y)            = exp(C) + exp(q_k) * (exp(K)-(1-deltak)*exp(K(-1))) + delta_kb * exp(K_b(-1));

% Eq. A39

exp(Y)              = exp(y_e); 

% Eq. A40

gamma_e * exp(l_pd) = gamma_p * exp(l_p);

% Eq. A41

gamma_e * exp(l_id) = gamma_i * exp(l_i);

% Eq. A42

h                   = gamma_p * exp(h_p) + gamma_i * exp(h_i); //

% Eq. A43

exp(BE)             = gamma_b * exp(b_e);

% Eq. A44

exp(BH)             = gamma_b * exp(b_h);

% Eq. A45

exp(L)              = exp(BH) + exp(BE) ;

% Eq. A46

exp(D)              = gamma_p * exp(d_p);

% Eq. A47

exp(K)              = gamma_e * exp(k_e);

% ***************
% Other variables
% ***************

exp(B)              = exp(weight_BH) * exp( BH ) + exp(weight_BE) * exp( BE );       % risk-weighted loans

exp(C)              = gamma_p * exp(c_p) + gamma_i * exp(c_i) + gamma_e * exp(c_e);	 % total consumption

exp(J_B)            = gamma_b * exp(j_B);  % banks' profits

exp(PIW)            = ( exp(w_p) + exp(w_i) )  / ( exp(w_p(-1)) + exp(w_i(-1)) ) * exp(pie);  % nominal wage inflation

exp(rr_e)           = exp(l_pd)^ni * exp(l_id)^(1-ni); 		% free variable

exp(aux1)           = exp(j_B);		                        % free variable

exp(k_ratio)        = exp(K_b) / ( exp(weight_BH) * exp(BH) + exp(weight_BE) * exp(BE) );

exp(spr_b)          = 0.5*exp(r_bh) + 0.5*exp(r_be) - exp(r_d);

exp(d_vi)           = exp(vi)/exp(vi(-1));

exp(d_r)            = exp(r_ib)/exp(r_ib(-1));

exp(L_Y)            = exp(L)/exp(Y1);		                       % loans-to-output ratio

%********************************************
%*********** EXOGENOUS PROCESSES ************
%********************************************

exp(ee_z)     = 1 - rho_ee_z   *    1   + rho_ee_z   * exp(ee_z(-1))    + e_z;

// Eq. A48

exp(A_e)      = 1 - rho_A_e    *    1   + rho_A_e    * exp(A_e(-1))     + e_A_e;	    // technology shock

// Eq. A49

exp(eps_K_b)  = (1-rho_eps_K_b)*    1   + rho_eps_K_b* exp(eps_K_b(-1)) + e_eps_K_b;    // financial shock

// Eq. A50

exp(ee_qk)    =  1-rho_ee_qk   *    1          + rho_ee_qk  * exp(ee_qk(-1))   + e_qk;  // investment-specific technology shock


exp(ee_j)     = 1 - rho_ee_j   *    1          + rho_ee_j   * exp(ee_j(-1))    + e_j;

exp(m_i)      = (1-rho_mi)     *  m_i_ss       + rho_mi     * exp(m_i(-1))     + e_mi;

exp(m_e)      = (1-rho_me)     *  m_e_ss       + rho_me     * exp(m_e(-1))     + e_me;

exp(mk_d)     = (1-rho_mk_d)   * mk_d_ss       + rho_mk_d   * exp(mk_d(-1))    + e_mk_d;

exp(mk_be)    = (1-rho_mk_be)  * mk_be_ss      + rho_mk_be  * exp(mk_be(-1))   + e_mk_be;

exp(mk_bh)    = (1-rho_mk_bh)  * mk_bh_ss      + rho_mk_bh  * exp(mk_bh(-1))   + e_mk_bh;

exp(eps_y)    = (1-rho_eps_y)  * eps_y_ss      + rho_eps_y  * exp(eps_y(-1))   + e_y;

exp(eps_l)    = (1-rho_eps_l)  * eps_l_ss      + rho_eps_l  * exp(eps_l(-1))   + e_l;


end; // End definition of the model

% // **********************************************************
% // setting initial conditions for computation of steady state
% // **********************************************************

initval;

c_p		     =	0.0470245	       ;
h_p		     =	-0.0491541	       ;
d_p		     =	1.33054	           ;
l_p		     =	-0.274717	       ;
lam_p	     =	-0.0470245	       ;
J_R		     =	-1.30888	       ;
j_B		     =	-3.14191	       ;
pie_wp	     =	-3.20E-11	       ;
c_i		     =	-1.77696	       ;
h_i		     =	-3.03727	       ;
b_i		     =	0.244596	       ;
l_i		     =	-0.055871	       ;
lam_i	     =	1.77696	           ;
s_i		     =	-2.86575	       ;
pie_wi	     =	-3.20E-11	       ;
I		     =	-1.54042	       ;
q_k		     =	1.95E-16	       ;
c_e		     =	-1.92458	       ;
k_e		     =	2.14846	           ;
l_pd	     =	-0.274717	       ;
l_id	     =	-0.055871	       ;
b_ee	     =	1.05783	           ;
y_e		     =	0.482875	       ;
lam_e	     =	1.92458	           ;
s_e		     =	-2.71813	       ;
d_b		     =	1.33054  	       ;
b_h		     =	0.244596	       ;
b_e		     =	1.05783	           ;
r_d		     =	-5.16157	       ;
r_bh	     =	-4.15995	       ;
r_be	     =	-4.15995	       ;
R_b		     =	-4.62258	       ;
K_b		     =	-0.983097	       ;
pie		     =	-3.20E-11	       ;
x		     =	0.182322	       ;
C		     =	0.309861	       ;
Y		     =	0.482875	       ;
D		     =	1.33054	           ;
BE		     =	1.05783	           ;
BH		     =	0.244596	       ;
B		     =	1.42485	           ;
w_p		     =	-0.0045485	       ;
w_i		     =	-1.60969	       ;
J_B		     =	-3.14191	       ;
q_h		     =	3.65403	           ;
K		     =	2.14846   	       ;
PIW		     =	-3.20E-11	       ;
r_ib	     =	-4.62258	       ;
r_k		     =	-3.05188	       ;
ee_z	     =	-2.35E-16	       ;
A_e		     =	-2.11E-15	       ;
ee_j	     =	-3.75E-16	       ;
mk_d	     =	-0.538997	       ;
mk_be	     =	0.462624	       ;
mk_bh	     =	0.462624	       ;
ee_qk	     =	4.90E-17	       ;
m_i		     =	-0.356675	       ;
m_e		     =	-1.04982	       ;
eps_y	     =	1.79176  	       ;
eps_l	     =	1.60944 	       ;
eps_K_b	     =	0        	       ;
Y1		     =	0.455859	       ;
spr_b	     =	-4.61769	       ;
rr_e	     =	-0.230947	       ;
aux1	     =	-3.14191	       ;
vi		     =	-2.40795	       ;
L		     =	1.42485	           ;
weight_BH    =	-9.31E-16	       ;
weight_BE	 =	-1.54E-16	       ;
P_S	         =	3.85269	           ;
k_ratio	     =	-2.40795	       ;
d_r		     =	0	               ;
d_vi	     =	0        	       ;
L_Y		     =	1.03476174831253   ;

end;

steady(solve_algo=0);
   
if ~exist([M_.fname '_steadystate.m'])
    write_ss_on_dynarefile;
    %     fprintf('\n...solve the SS a second time to check the _steadystate file...')
    %     steady(solve_algo=0);
end

check;

% ******************************
% define shocks used in solution
% ******************************

shocks;

var e_A_e       = 0.0062^2;  // std. dev. technology shocks

end;

// ************************
// Paramneters for solution
// ************************

options_.nograph         = 1;
options_.nomoments       = 0;
options_.noprint         = 1;
options_.nocorr          = 1;
options_.ar              = 0;
options_.order           = 1;
options_.moments_varendo = 1;

% ***************************
% Start solution of the model
% ***************************

[dr]         = resol(oo_.steady_state,1);
eigenvalues_ = dr.eigval;
nyf          = nnz(dr.kstate(:,2)>M_.maximum_lag+1);
[m_lambda,i] = sort(abs(eigenvalues_));
n_explod     = nnz(abs(eigenvalues_) > options_.qz_criterium);

% ***************************
% Check solution of the model
% ***************************

if oo_.dr.rank == oo_.dr.nsfwrd & oo_.dr.nsfwrd == n_explod & rho_vi < 0.999 & rho_ib < 0.999;

% ***********************************************************************************
% Select variables for computing solution, impulse responses and asymptotic variances
% ***********************************************************************************

stoch_simul(order=1) K_b  vi  Y1  C  I  B  q_h  q_k  J_B  BE  BH  k_ratio   L  pie  r_ib  d_vi  d_r  L_Y  R_b  r_d k_ratio r_be r_bh;
%                      1   2   3  4  5  6    7    8    9  10  11       12  13   14    15    16   17   18   19   20      21   22   23

% ******************************
% compute value of loss function
% ******************************

loss = [ 0.0*oo_.var(3,3)*100 + 1.0*oo_.var(18,18)*100 + 0.1*oo_.var(16,16)*100 ];
    
 else 

      loss = 1e5;
       
end

loss_function_fsp = loss;
